==> bribery-0.3.0%2B1.21.4-fabric-sources.jar: dev/spagurder/bribery/core/BribeHandler.java <==
package dev.spagurder.bribery.core;

import dev.spagurder.bribery.config.Config;
import dev.spagurder.bribery.config.CurrencyConfig;
import dev.spagurder.bribery.config.EntityConfig;
import dev.spagurder.bribery.state.BribeData;
import dev.spagurder.bribery.state.BriberyState;
import java.util.Random;
import net.minecraft.class_1268;
import net.minecraft.class_1303;
import net.minecraft.class_1309;
import net.minecraft.class_1439;
import net.minecraft.class_1646;
import net.minecraft.class_1799;
import net.minecraft.class_2398;
import net.minecraft.class_2561;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3417;
import net.minecraft.class_4139;
import net.minecraft.class_5354;

public class BribeHandler {

    public static boolean handle(class_1309 entity, class_3222 player, class_1799 bribe) {
        EntityConfig ec;
        if (entity instanceof class_1646) {
            ec = new EntityConfig(EntityConfig.VILLAGER);
        } else if (entity instanceof class_1439) {
            ec = new EntityConfig(EntityConfig.GOLEM);
        } else {
            return true;
        }

        CurrencyConfig cc = Config.CURRENCY_CONFIGS.get(bribe.method_7909());
        if (cc.bribeCredit <= 0) {
            if (Config.verbose) {
                player.method_7353(class_2561.method_43470("This currency is not enabled for bribing."), true);
            }
            return false;
        }

        if (!ec.bribable()) {
            if (Config.verbose) {
                player.method_7353(class_2561.method_43470("Entity is not bribable."), true);
            }
            return false;
        }

        BribeData state = BriberyState.getOrCreateBribeData(entity.method_5667(), player.method_5667());
        float credit = cc.bribeCredit * bribe.method_7947();
        state.largestBribe = (int) Math.ceil(Math.max(state.largestBribe, credit));
        long gameTime = BriberyUtil.overworldGameTime(entity.method_5682());

        if (state.isExtorting) {
            extort(entity, player, state, credit);
            return false;
        } else if (state.isCoolingDown) {
            if (state.isBribed) {
                long delta = gameTime - state.bribedAt;
                if (delta >= 0 && delta < Config.acceptedCooldownSeconds * 20L) {
                    if (Config.verbose) {
                        player.method_7353(
                                class_2561.method_43470("This entity cannot accept bribes right now."), true
                        );
                    }
                    return false;
                }
            }
            if (state.isRejected) {
                long delta = gameTime - state.rejectedAt;
                if (delta >= 0 && delta < Config.rejectedCooldownSeconds * 20L) {
                    if (Config.rejectedCooldownAllowAttempts) {
                        reject(entity, player, state, credit);
                        return false;
                    }
                    if (Config.verbose) {
                        player.method_7353(
                                class_2561.method_43470("This entity cannot accept bribes right now."), true
                        );
                    }
                }
                state.isRejected = false;
            }
            state.isCoolingDown = false;
        } else {
            state.isRejected = false;
        }

        if (Config.bribeExpiryMinutes > 0) {
            long delta = gameTime - state.bribedAt;
            if (delta < 0 || delta >= Config.bribeExpiryMinutes * 1200L) {
                state.isBribed = false;
            }
        }

        Random random = new Random();
        if (!state.isExtortionist) {
            float rejectionChance = Math.max(
                    ec.rejectionChance() - (cc.rejectionChanceModifier * bribe.method_7947()),
                    ec.minimumRejectionChance()
            );
            if (state.isBribed) {
                rejectionChance *= Config.alreadyBribedMultiplier;
            }
            if (random.nextFloat() < (rejectionChance / 100f)) {
                reject(entity, player, state, credit);
                return false;
            }
        }
        accept(entity, player, state, credit, random);

        return false;
    }

    public static void accept(class_1309 entity, class_3222 player, BribeData state, float credit, Random random) {
        if (!state.isExtortionist) {
            if (random.nextFloat() < (Config.extortionistChance / 100f)) {
                state.isExtortionist = true;
            }
        }
        state.isBribed = true;
        state.isCoolingDown = true;
        state.bribedAt = BriberyUtil.overworldGameTime(entity.method_5682());
        state.bribeCredits += (int) Math.ceil(credit);
        if (Config.bribeXpMultiplier > 0) {
            int totalXp = (int) Math.ceil(credit * Config.bribeXpMultiplier);
            entity.method_37908().method_8649(
                    new class_1303(entity.method_37908(), entity.method_23317(), entity.method_23318(), entity.method_23321(), totalXp)
            );
        }
        if (entity instanceof class_1646 villager) {
            villager.method_43077(class_3417.field_14815);
            ((class_3218) villager.method_37908()).method_65096(
                    class_2398.field_11211,
                    villager.method_23317(), villager.method_23318() + 1.0, villager.method_23321(),
                    5, 0.5, 0.5, 0.5, 0
            );
            villager.method_21651().method_35124(player.method_5667(), class_4139.field_18424);
            villager.method_21651().method_35124(player.method_5667(), class_4139.field_18425);
            if (Config.acceptedGossipMultiplier > 0) {
                villager.method_21651()
                        .method_19072(
                                player.method_5667(),
                                class_4139.field_18428,
                                (int) (credit * Config.acceptedGossipMultiplier)
                        );
            }
        }
        player.method_6122(class_1268.field_5808, class_1799.field_8037);
    }

    public static void reject(class_1309 entity, class_3222 player, BribeData state, float credit) {
        state.isExtortionist = false;
        state.isExtorting = false;
        state.isBribed = false;
        state.isCoolingDown = true;
        state.isRejected = true;
        state.rejectedAt = BriberyUtil.overworldGameTime(entity.method_5682());
        if (entity instanceof class_1646 villager) {
            villager.method_43077(class_3417.field_15008);
            ((class_3218) villager.method_37908()).method_65096(
                    class_2398.field_11231,
                    villager.method_23317(), villager.method_23318() + 1.0, villager.method_23321(),
                    5, 0.5, 0.5, 0.5, 0
            );
            if (Config.rejectedGossipMultiplier > 0) {
                villager.method_21651()
                        .method_19072(
                                player.method_5667(),
                                class_4139.field_18424,
                                (int) (credit * Config.rejectedGossipMultiplier)
                        );
                villager.method_6015(player);
            }
        }
        if (entity instanceof class_5354 mob) {
            mob.method_5980(player);
            mob.method_29513(player.method_5667());
            mob.method_29514(12000);
        }
    }

    public static void extort(class_1309 entity, class_3222 player, BribeData state, float credit) {
        state.extortionBalance = Math.max(
                0,
                state.extortionBalance - (int) Math.ceil(credit * Config.extortionPriceMultiplier)
        );
        if (state.extortionBalance > 0) {
            player.method_7353(
                    class_2561.method_43470(entity.method_5476() + " is demanding more payment."),
                    true
            );
        } else {
            player.method_7353(
                    class_2561.method_43470(entity.method_5476() + " is satisfied... for now."),
                    true
            );
            state.isExtorting = false;
            long gameTime = BriberyUtil.overworldGameTime(entity.method_5682());
            state.extortedAt = gameTime;
            state.bribedAt = gameTime;
        }
    }

    public static void cancel(class_1309 entity, class_3222 player) {
        BribeData state = BriberyState.getBribeData(entity.method_5667(), player.method_5667());
        if (state == null) return;
        if (state.isExtorting) {
            reject(entity, player, state, state.largestBribe * Config.extortionPriceMultiplier);
            return;
        }
        BriberyState.bribeStates.get(entity.method_5667()).remove(player.method_5667());
    }

}

==> bribery-0.4.0%2B1.20.4-fabric-sources.jar: dev/spagurder/bribery/core/BribeHandler.java <==
package dev.spagurder.bribery.core;

import dev.spagurder.bribery.config.Config;
import dev.spagurder.bribery.config.CurrencyConfig;
import dev.spagurder.bribery.config.EntityConfig;
import dev.spagurder.bribery.state.BribeData;
import dev.spagurder.bribery.state.BriberyState;
import java.util.Random;
import net.minecraft.class_1268;
import net.minecraft.class_1303;
import net.minecraft.class_1309;
import net.minecraft.class_1439;
import net.minecraft.class_1646;
import net.minecraft.class_1799;
import net.minecraft.class_2398;
import net.minecraft.class_2561;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3417;
import net.minecraft.class_4139;
import net.minecraft.class_5354;

public class BribeHandler {

    public static boolean handle(class_1309 entity, class_3222 player, class_1799 bribe) {
        EntityConfig ec;
        if (entity instanceof class_1646) {
            ec = new EntityConfig(EntityConfig.VILLAGER);
        } else if (entity instanceof class_1439) {
            ec = new EntityConfig(EntityConfig.GOLEM);
        } else {
            return true;
        }
        witnessBribe(entity, player);

        CurrencyConfig cc = Config.CURRENCY_CONFIGS.get(bribe.method_7909());
        if (cc.bribeCredit <= 0) {
            if (Config.verbose) {
                player.method_7353(class_2561.method_43470("This currency is not enabled for bribing."), true);
            }
            return false;
        }

        if (!ec.bribable()) {
            if (Config.verbose) {
                player.method_7353(class_2561.method_43470("Entity is not bribable."), true);
            }
            return false;
        }

        BribeData state = BriberyState.getOrCreateBribeData(entity.method_5667(), player.method_5667());
        float credit = cc.bribeCredit * bribe.method_7947();
        state.largestBribe = (int) Math.ceil(Math.max(state.largestBribe, credit));
        long gameTime = BriberyUtil.overworldGameTime(entity.method_5682());

        if (state.isExtorting) {
            extort(entity, player, state, credit);
            return false;
        } else if (state.isCoolingDown) {
            if (state.isBribed) {
                long delta = gameTime - state.bribedAt;
                if (delta >= 0 && delta < Config.acceptedCooldownSeconds * 20L) {
                    if (Config.verbose) {
                        player.method_7353(
                                class_2561.method_43470("This entity cannot accept bribes right now."), true
                        );
                    }
                    return false;
                }
            }
            if (state.isRejected) {
                long delta = gameTime - state.rejectedAt;
                if (delta >= 0 && delta < Config.rejectedCooldownSeconds * 20L) {
                    if (Config.rejectedCooldownAllowAttempts) {
                        reject(entity, player, state, credit);
                        return false;
                    }
                    if (Config.verbose) {
                        player.method_7353(
                                class_2561.method_43470("This entity cannot accept bribes right now."), true
                        );
                    }
                }
                state.isRejected = false;
            }
            state.isCoolingDown = false;
        } else {
            state.isRejected = false;
        }

        if (Config.bribeExpiryMinutes > 0) {
            long delta = gameTime - state.bribedAt;
            if (delta < 0 || delta >= Config.bribeExpiryMinutes * 1200L) {
                state.isBribed = false;
            }
        }

        Random random = new Random();
        if (!state.isExtortionist) {
            float rejectionChance = Math.max(
                    ec.rejectionChance() - (cc.rejectionChanceModifier * bribe.method_7947()),
                    ec.minimumRejectionChance()
            );
            if (state.isBribed) {
                rejectionChance *= Config.alreadyBribedMultiplier;
            }
            if (entity instanceof class_5354 mob) {
                if (mob.method_29511()) {
                    rejectionChance += Config.aggroRejectionModifier;
                }
            }
            if (random.nextFloat() < (rejectionChance / 100f)) {
                reject(entity, player, state, credit);
                return false;
            }
        }
        accept(entity, player, state, credit, random);

        return false;
    }

    public static void accept(class_1309 entity, class_3222 player, BribeData state, float credit, Random random) {
        long gameTime = BriberyUtil.overworldGameTime(entity.method_5682());
        if (!state.isExtortionist) {
            if (random.nextFloat() < (Config.extortionistChance / 100f)) {
                state.isExtortionist = true;
                state.extortedAt = gameTime;
            }
        }
        state.isBribed = true;
        state.isCoolingDown = true;
        state.bribedAt = gameTime;
        state.bribeCredits += (int) Math.ceil(credit);
        if (Config.bribeXpMultiplier > 0) {
            int totalXp = (int) Math.ceil(credit * Config.bribeXpMultiplier);
            entity.method_37908().method_8649(
                    new class_1303(entity.method_37908(), entity.method_23317(), entity.method_23318(), entity.method_23321(), totalXp)
            );
        }
        if (entity instanceof class_5354 mob) {
            if (mob.method_29511()) {
                mob.method_5980(null);
                mob.method_29513(null);
                mob.method_29514(0);
            }
        }
        if (entity instanceof class_1646 villager) {
            villager.method_43077(class_3417.field_14815);
            ((class_3218) villager.method_37908()).method_14199(
                    class_2398.field_11211,
                    villager.method_23317(), villager.method_23318() + 1.0, villager.method_23321(),
                    5, 0.5, 0.5, 0.5, 0
            );
            villager.method_21651().method_35124(player.method_5667(), class_4139.field_18424);
            villager.method_21651().method_35124(player.method_5667(), class_4139.field_18425);
            if (Config.acceptedGossipMultiplier > 0) {
                villager.method_21651()
                        .method_19072(
                                player.method_5667(),
                                class_4139.field_18428,
                                (int) (credit * Config.acceptedGossipMultiplier)
                        );
            }
        }
        player.method_6122(class_1268.field_5808, class_1799.field_8037);
    }

    public static void reject(class_1309 entity, class_3222 player, BribeData state, float credit) {
        state.isExtortionist = false;
        state.isExtorting = false;
        state.isBribed = false;
        state.isCoolingDown = true;
        state.isRejected = true;
        state.rejectedAt = BriberyUtil.overworldGameTime(entity.method_5682());
        if (entity instanceof class_1646 villager) {
            villager.method_43077(class_3417.field_15008);
            ((class_3218) villager.method_37908()).method_14199(
                    class_2398.field_11231,
                    villager.method_23317(), villager.method_23318() + 1.0, villager.method_23321(),
                    5, 0.5, 0.5, 0.5, 0
            );
            if (Config.rejectedGossipMultiplier > 0) {
                villager.method_21651()
                        .method_19072(
                                player.method_5667(),
                                class_4139.field_18424,
                                (int) (credit * Config.rejectedGossipMultiplier)
                        );
                villager.method_6015(player);
            }
        }
        if (entity instanceof class_5354 mob) {
            BriberyUtil.makeMobAngry(mob, player);
        }
    }

    public static void extort(class_1309 entity, class_3222 player, BribeData state, float credit) {
        state.extortionBalance = Math.max(
                0,
                state.extortionBalance - (int) Math.ceil(credit * Config.extortionPriceMultiplier)
        );
        if (state.extortionBalance > 0) {
            player.method_7353(
                    class_2561.method_43473().method_10852(entity.method_5476()).method_27693(" is demanding more payment."),
                    true
            );
        } else {
            player.method_7353(
                    class_2561.method_43473().method_10852(entity.method_5476()).method_27693(" is satisfied... for now."),
                    true
            );
            state.isExtorting = false;
            long gameTime = BriberyUtil.overworldGameTime(entity.method_5682());
            state.extortedAt = gameTime;
            state.bribedAt = gameTime;
        }
    }

    public static void cancel(class_1309 entity, class_3222 player) {
        BribeData state = BriberyState.getBribeData(entity.method_5667(), player.method_5667());
        if (state == null) return;
        if (state.isRejected) return;
        if (state.isExtorting) {
            reject(entity, player, state, state.largestBribe * Config.extortionPriceMultiplier);
            return;
        }
        BriberyState.bribeStates.get(entity.method_5667()).remove(player.method_5667());
    }

    public static void witnessBribe(class_1309 entity, class_3222 player) {
        if (Config.witnessBribeRange <= 0) return;
        entity.method_37908().method_8390(
                class_1309.class,
                entity.method_5829().method_1014(Config.witnessBribeRange),
                e -> e != entity && BriberyUtil.isValidWitness(e) && e.method_6057(player)
        ).forEach(witness -> {
            BribeData state = BriberyState.getBribeData(witness.method_5667(), player.method_5667());
            if (state == null || !state.isBribed) {
                if (BriberyUtil.inFOV(witness, player, Config.witnessBribeFovDegrees)) {
                    if (witness instanceof class_1646 villager) {
                        villager.method_21651().method_19072(player.method_5667(), class_4139.field_18427, 25);
                    }
                    if (witness instanceof class_5354 mob) {
                        BriberyUtil.makeMobAngry(mob, player);
                    }
                }
            }
        });
    }

}
